@@ -13,11 +13,11 @@ use 5.010001;
 
 no warnings qw( threads recursion uninitialized numeric );
 
-our $VERSION = '1.824';
+our $VERSION = '1.825';
 
-use MCE::Shared::Base;
+use MCE::Shared::Base ();
 use MCE::Util ();
-use MCE::Mutex;
+use MCE::Mutex ();
 use bytes;
 
 use overload (
@@ -135,7 +135,7 @@ MCE::Shared::Condvar - Condvar helper class
 
 =head1 VERSION
 
-This document describes MCE::Shared::Condvar version 1.824
+This document describes MCE::Shared::Condvar version 1.825
 
 =head1 DESCRIPTION
 
