@@ -25,12 +25,12 @@ use 5.010001;
 
 no warnings qw( threads recursion uninitialized numeric );
 
-our $VERSION = '1.824';
+our $VERSION = '1.825';
 
 ## no critic (Subroutines::ProhibitExplicitReturnUndef)
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
-use MCE::Shared::Base;
+use MCE::Shared::Base ();
 use base 'MCE::Shared::Base::Common';
 use bytes;
 
@@ -326,14 +326,15 @@ sub SPLICE {
 
    $self->purge if %{ $self->[_INDX] };
 
-   my ( $key, @ret );
    my $size = scalar @{ $keys };
    my $len  = @_ ? shift : $size - $off;
+   my @ret;
 
    if ( $off >= $size ) {
       $self->push( @_ ) if @_;
    }
    elsif ( abs($off) <= $size ) {
+      local $_;
       if ( $len > 0 ) {
          $off = $off + @{ $keys } if ( $off < 0 );
          my @k = splice @{ $keys }, $off, $len;
@@ -797,7 +798,7 @@ MCE::Shared::Ordhash - An ordered hash class featuring tombstone deletion
 
 =head1 VERSION
 
-This document describes MCE::Shared::Ordhash version 1.824
+This document describes MCE::Shared::Ordhash version 1.825
 
 =head1 DESCRIPTION
 
