@@ -13,11 +13,11 @@ use 5.010001;
 
 no warnings qw( threads recursion uninitialized numeric );
 
-our $VERSION = '1.824';
+our $VERSION = '1.825';
 
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
-use MCE::Shared::Base;
+use MCE::Shared::Base ();
 use bytes;
 
 use overload (
@@ -101,7 +101,7 @@ MCE::Shared::Scalar - Scalar helper class
 
 =head1 VERSION
 
-This document describes MCE::Shared::Scalar version 1.824
+This document describes MCE::Shared::Scalar version 1.825
 
 =head1 DESCRIPTION
 
